@@ -181,10 +181,9 @@ bool ColorSchemeManager::deleteColorScheme(const QString &name)
     if (QFile::remove(path)) {
         delete _colorSchemes.take(name);
         return true;
-    } else {
-        qCDebug(KonsoleDebug)<<"Failed to remove color scheme -"<<path;
-        return false;
-    }
+    } 
+    qCDebug(KonsoleDebug)<<"Failed to remove color scheme -"<<path;
+    return false;
 }
 
 const ColorScheme *ColorSchemeManager::findColorScheme(const QString &name)
@@ -203,17 +202,16 @@ const ColorScheme *ColorSchemeManager::findColorScheme(const QString &name)
 
     if (_colorSchemes.contains(name)) {
         return _colorSchemes[name];
-    } else {
-        // look for this color scheme
-        QString path = findColorSchemePath(name);
-        if (!path.isEmpty() && loadColorScheme(path)) {
-            return findColorScheme(name);
-        }
-
-        qCDebug(KonsoleDebug) << "Could not find color scheme - " << name;
-
-        return nullptr;
+    } 
+    // look for this color scheme
+    QString path = findColorSchemePath(name);
+    if (!path.isEmpty() && loadColorScheme(path)) {
+        return findColorScheme(name);
     }
+
+    qCDebug(KonsoleDebug) << "Could not find color scheme - " << name;
+
+    return nullptr;
 }
 
 QString ColorSchemeManager::findColorSchemePath(const QString &name) const
