@@ -28,11 +28,23 @@
 #include <QStringList>
 #include <QRegularExpression>
 #include <QMultiHash>
+#include <QRect>
+#include <QPoint>
+
+// KDE
+#include <KFileItemActions>
+#include <KFileItem>
+#include <KIO/PreviewJob>
+
+#include <memory>
 
 // Konsole
 #include "Character.h"
 
 class QAction;
+class QMenu;
+class QMouseEvent;
+class KFileItem;
 
 namespace Konsole {
 class Session;
@@ -72,6 +84,10 @@ public:
     */
     class HotSpot : public QObject
     {
+        // krazy suggest using Q_OBJECT here but moc can not handle
+        // nested classes
+        // QObject derived classes should use the Q_OBJECT macro
+
     public:
         /**
          * Constructs a new hotspot which covers the area from (@p startLine,@p startColumn) to (@p endLine,@p endColumn)
@@ -85,6 +101,8 @@ public:
             NotSpecified,
             // this hotspot represents a clickable link
             Link,
+            // this hotspot represents a clickable e-mail address
+            EMailAddress,
             // this hotspot represents a marker
             Marker
         };
@@ -117,6 +135,10 @@ public:
          */
         virtual QList<QAction *> actions();
 
+        /**
+         * Setups a menu with actions for the hotspot.
+         */
+        virtual void setupMenu(QMenu *menu);
     protected:
         /** Sets the type of a hotspot.  This should only be set once */
         void setType(Type type);
@@ -161,7 +183,7 @@ protected:
     /** Returns the internal buffer */
     const QString *buffer();
     /** Converts a character position within buffer() to a line and column */
-    void getLineColumn(int position, int &startLine, int &startColumn);
+    std::pair<int,int> getLineColumn(int position);
 
 private:
     Q_DISABLE_COPY(Filter)
@@ -301,9 +323,30 @@ public:
          * Opens kate for editing the file.
          */
         void activate(QObject *object = nullptr) override;
+        void setupMenu(QMenu *menu) override;
 
+        KFileItem fileItem() const;
+        void requestThumbnail(Qt::KeyboardModifiers modifiers, const QPoint &pos);
+        void thumbnailRequested();
+
+        static void stopThumbnailGeneration();
     private:
+        void showThumbnail(const KFileItem& item, const QPixmap& preview);
         QString _filePath;
+        KFileItemActions _menuActions;
+
+        QPoint _eventPos;
+        QPoint _thumbnailPos;
+        Qt::KeyboardModifiers _eventModifiers;
+        bool _thumbnailFinished;
+
+        /* This variable stores the pointer of the active HotSpot that
+         * is generating the thumbnail now, so we can bail out early.
+         * it's not used for pointer access.
+         */
+        static qintptr currentThumbnailHotspot;
+        static bool _canGenerateThumbnail;
+        static QPointer<KIO::PreviewJob> _previewJob;
     };
 
     explicit FileFilter(Session *session);
@@ -316,7 +359,7 @@ protected:
 private:
     QPointer<Session> _session;
     QString _dirPath;
-    QSet<QString> _currentFiles;
+    QSet<QString> _currentDirContents;
 };
 
 /**
@@ -336,7 +379,7 @@ private:
  * The hotSpots() method return all of the hotspots in the text and on
  * a given line respectively.
  */
-class FilterChain : protected QList<Filter *>
+class FilterChain
 {
 public:
     virtual ~FilterChain();
@@ -362,6 +405,8 @@ public:
     QSharedPointer<Filter::HotSpot> hotSpotAt(int line, int column) const;
     /** Returns a list of all the hotspots in all the chain's filters */
     QList<QSharedPointer<Filter::HotSpot>> hotSpots() const;
+protected:
+    QList<Filter *> _filters;
 };
 
 /** A filter chain which processes character images from terminal displays */
@@ -385,8 +430,10 @@ public:
 private:
     Q_DISABLE_COPY(TerminalImageFilterChain)
 
-    QString *_buffer;
-    QList<int> *_linePositions;
+/* usually QStrings and QLists are not supposed to be in the heap, here we have a problem:
+    we need a shared memory space between many filter objeccts, defined by this TerminalImage. */
+    std::unique_ptr<QString> _buffer;
+    std::unique_ptr<QList<int>> _linePositions;
 };
 }
 #endif //FILTER_H
