@@ -56,6 +56,7 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KCodecAction>
+#include <KNotification>
 
 // Konsole
 #include "EditProfileDialog.h"
@@ -75,19 +76,13 @@
 #include "PrintOptions.h"
 #include "SaveHistoryTask.h"
 #include "SearchHistoryTask.h"
+#include "SessionGroup.h"
 
 // For Unix signal names
 #include <csignal>
 
 using namespace Konsole;
 
-// TODO - Replace the icon choices below when suitable icons for silence and
-// activity are available
-Q_GLOBAL_STATIC_WITH_ARGS(QIcon, _activityIcon, (QIcon::fromTheme(QLatin1String("dialog-information"))))
-Q_GLOBAL_STATIC_WITH_ARGS(QIcon, _silenceIcon, (QIcon::fromTheme(QLatin1String("dialog-information"))))
-Q_GLOBAL_STATIC_WITH_ARGS(QIcon, _bellIcon, (QIcon::fromTheme(QLatin1String("preferences-desktop-notification-bell"))))
-Q_GLOBAL_STATIC_WITH_ARGS(QIcon, _broadcastIcon, (QIcon::fromTheme(QLatin1String("emblem-important"))))
-
 QSet<SessionController*> SessionController::_allControllers;
 int SessionController::_lastControllerId;
 
@@ -100,7 +95,6 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     , _profileList(nullptr)
     , _sessionIcon(QIcon())
     , _sessionIconName(QString())
-    , _previousState(-1)
     , _searchFilter(nullptr)
     , _urlFilter(nullptr)
     , _fileFilter(nullptr)
@@ -116,13 +110,13 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     , _webSearchMenu(nullptr)
     , _listenForScreenWindowUpdates(false)
     , _preventClose(false)
-    , _keepIconUntilInteraction(false)
     , _selectedText(QString())
     , _showMenuAction(nullptr)
     , _bookmarkValidProgramsToClear(QStringList())
     , _isSearchBarEnabled(false)
     , _editProfileDialog(nullptr)
     , _searchBar(view->searchBar())
+    , _monitorProcessFinish(false)
 {
     Q_ASSERT(session);
     Q_ASSERT(view);
@@ -148,8 +142,8 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     setIdentifier(++_lastControllerId);
     sessionAttributeChanged();
 
-    view->installEventFilter(this);
-    view->setSessionController(this);
+    connect(_view, &TerminalDisplay::compositeFocusChanged, this, &SessionController::viewFocusChangeHandler);
+    _view->setSessionController(this);
 
     // install filter on the view to highlight URLs and files
     updateFilterList(SessionManager::instance()->sessionProfile(_session));
@@ -158,40 +152,41 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     connect(ProfileManager::instance(), &Konsole::ProfileManager::profileChanged, this, &Konsole::SessionController::updateFilterList);
 
     // listen for session resize requests
-    connect(_session.data(), &Konsole::Session::resizeRequest, this, &Konsole::SessionController::sessionResizeRequest);
+    connect(_session, &Konsole::Session::resizeRequest, this, &Konsole::SessionController::sessionResizeRequest);
 
     // listen for popup menu requests
-    connect(_view.data(), &Konsole::TerminalDisplay::configureRequest, this, &Konsole::SessionController::showDisplayContextMenu);
+    connect(_view, &Konsole::TerminalDisplay::configureRequest, this, &Konsole::SessionController::showDisplayContextMenu);
 
     // move view to newest output when keystrokes occur
-    connect(_view.data(), &Konsole::TerminalDisplay::keyPressedSignal, this, &Konsole::SessionController::trackOutput);
+    connect(_view, &Konsole::TerminalDisplay::keyPressedSignal, this, &Konsole::SessionController::trackOutput);
 
     // listen to activity / silence notifications from session
-    connect(_session.data(), &Konsole::Session::stateChanged, this, &Konsole::SessionController::sessionStateChanged);
+    connect(_session, &Konsole::Session::notificationsChanged, this, &Konsole::SessionController::sessionNotificationsChanged);
     // listen to title and icon changes
-    connect(_session.data(), &Konsole::Session::sessionAttributeChanged, this, &Konsole::SessionController::sessionAttributeChanged);
-    connect(_session.data(), &Konsole::Session::readOnlyChanged, this, &Konsole::SessionController::sessionReadOnlyChanged);
+    connect(_session, &Konsole::Session::sessionAttributeChanged, this, &Konsole::SessionController::sessionAttributeChanged);
+    connect(_session, &Konsole::Session::readOnlyChanged, this, &Konsole::SessionController::sessionReadOnlyChanged);
 
-    connect(this, &Konsole::SessionController::tabRenamedByUser,  _session,  &Konsole::Session::tabTitleSetByUser);
+    connect(this, &Konsole::SessionController::tabRenamedByUser, _session, &Konsole::Session::tabTitleSetByUser);
+    connect(this, &Konsole::SessionController::tabColoredByUser, _session, &Konsole::Session::tabColorSetByUser);
 
-    connect(_session.data() , &Konsole::Session::currentDirectoryChanged , this , &Konsole::SessionController::currentDirectoryChanged);
+    connect(_session , &Konsole::Session::currentDirectoryChanged , this , &Konsole::SessionController::currentDirectoryChanged);
 
     // listen for color changes
-    connect(_session.data(), &Konsole::Session::changeBackgroundColorRequest, _view.data(), &Konsole::TerminalDisplay::setBackgroundColor);
-    connect(_session.data(), &Konsole::Session::changeForegroundColorRequest, _view.data(), &Konsole::TerminalDisplay::setForegroundColor);
+    connect(_session, &Konsole::Session::changeBackgroundColorRequest, _view, &Konsole::TerminalDisplay::setBackgroundColor);
+    connect(_session, &Konsole::Session::changeForegroundColorRequest, _view, &Konsole::TerminalDisplay::setForegroundColor);
 
     // update the title when the session starts
-    connect(_session.data(), &Konsole::Session::started, this, &Konsole::SessionController::snapshot);
+    connect(_session, &Konsole::Session::started, this, &Konsole::SessionController::snapshot);
 
     // listen for output changes to set activity flag
     connect(_session->emulation(), &Konsole::Emulation::outputChanged, this, &Konsole::SessionController::fireActivity);
 
     // listen for detection of ZModem transfer
-    connect(_session.data(), &Konsole::Session::zmodemDownloadDetected, this, &Konsole::SessionController::zmodemDownload);
-    connect(_session.data(), &Konsole::Session::zmodemUploadDetected, this, &Konsole::SessionController::zmodemUpload);
+    connect(_session, &Konsole::Session::zmodemDownloadDetected, this, &Konsole::SessionController::zmodemDownload);
+    connect(_session, &Konsole::Session::zmodemUploadDetected, this, &Konsole::SessionController::zmodemUpload);
 
     // listen for flow control status changes
-    connect(_session.data(), &Konsole::Session::flowControlEnabledChanged, _view.data(), &Konsole::TerminalDisplay::setFlowControlWarningEnabled);
+    connect(_session, &Konsole::Session::flowControlEnabledChanged, _view, &Konsole::TerminalDisplay::setFlowControlWarningEnabled);
     _view->setFlowControlWarningEnabled(_session->flowControlEnabled());
 
     // take a snapshot of the session state every so often when
@@ -203,8 +198,10 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     _interactionTimer->setSingleShot(true);
     _interactionTimer->setInterval(500);
     connect(_interactionTimer, &QTimer::timeout, this, &Konsole::SessionController::snapshot);
-    connect(_view.data(), &Konsole::TerminalDisplay::focusGained, this, &Konsole::SessionController::interactionHandler);
-    connect(_view.data(), &Konsole::TerminalDisplay::keyPressedSignal, this, &Konsole::SessionController::interactionHandler);
+    connect(_view, &Konsole::TerminalDisplay::compositeFocusChanged,
+            this, [this](bool focused) { if (focused) { interactionHandler(); }});
+    connect(_view, &Konsole::TerminalDisplay::keyPressedSignal,
+            this, &Konsole::SessionController::interactionHandler);
 
     // take a snapshot of the session state periodically in the background
     auto backgroundTimer = new QTimer(_session);
@@ -214,18 +211,24 @@ SessionController::SessionController(Session* session , TerminalDisplay* view, Q
     backgroundTimer->start();
 
     // xterm '10;?' request
-    connect(_session.data(), &Konsole::Session::getForegroundColor,
+    connect(_session, &Konsole::Session::getForegroundColor,
             this, &Konsole::SessionController::sendForegroundColor);
     // xterm '11;?' request
-    connect(_session.data(), &Konsole::Session::getBackgroundColor,
+    connect(_session, &Konsole::Session::getBackgroundColor,
             this, &Konsole::SessionController::sendBackgroundColor);
 
     _allControllers.insert(this);
 
     // A list of programs that accept Ctrl+C to clear command line used
     // before outputting bookmark.
-    _bookmarkValidProgramsToClear << QStringLiteral("bash") << QStringLiteral("fish") << QStringLiteral("sh");
-    _bookmarkValidProgramsToClear << QStringLiteral("tcsh") << QStringLiteral("zsh");
+    _bookmarkValidProgramsToClear = QStringList({
+        QStringLiteral("bash"),
+        QStringLiteral("fish"),
+        QStringLiteral("sh"),
+        QStringLiteral("tcsh"),
+        QStringLiteral("zsh")
+    });
+
     setupSearchBar();
     _searchBar->setVisible(_isSearchBarEnabled);
 }
@@ -235,9 +238,9 @@ SessionController::~SessionController()
     _allControllers.remove(this);
 
     if (!_editProfileDialog.isNull()) {
-        delete _editProfileDialog.data();
+        _editProfileDialog->deleteLater();
     }
-    if(factory()) {
+    if(factory() != nullptr) {
         factory()->removeClient(this);
     }
 }
@@ -271,13 +274,32 @@ void SessionController::trackOutput(QKeyEvent* event)
 
     _view->screenWindow()->setTrackOutput(true);
 }
+
+void SessionController::viewFocusChangeHandler(bool focused)
+{
+    if (focused) {
+        // notify the world that the view associated with this session has been focused
+        // used by the view manager to update the title of the MainWindow widget containing the view
+        emit viewFocused(this);
+
+        // when the view is focused, set bell events from the associated session to be delivered
+        // by the focused view
+
+        // first, disconnect any other views which are listening for bell signals from the session
+        disconnect(_session, &Konsole::Session::bellRequest, nullptr, nullptr);
+        // second, connect the newly focused view to listen for the session's bell signal
+        connect(_session, &Konsole::Session::bellRequest, _view, &Konsole::TerminalDisplay::bell);
+
+        if ((_copyInputToAllTabsAction != nullptr) && _copyInputToAllTabsAction->isChecked()) {
+            // A session with "Copy To All Tabs" has come into focus:
+            // Ensure that newly created sessions are included in _copyToGroup!
+            copyInputToAllTabs();
+        }
+    }
+}
+
 void SessionController::interactionHandler()
 {
-    // This flag is used to make sure those special icons indicating interest
-    // events (activity/silence/bell?) remain in the tab until user interaction
-    // happens. Otherwise, those special icons will quickly be replaced by
-    // normal icon when ::snapshot() is triggered
-    _keepIconUntilInteraction = false;
     _interactionTimer->start();
 }
 
@@ -298,9 +320,31 @@ void SessionController::snapshot()
         title = _session->title(Session::NameRole);
     }
 
+    QColor color = _session->color();
+    // use the fallback color if needed
+    if (!color.isValid()) {
+        color = QColor(QColor::Invalid);
+    }
+
     // apply new title
     _session->setTitle(Session::DisplayedTitleRole, title);
 
+    // apply new color
+    _session->setColor(color);
+
+    // check if foreground process ended and notify if this option was requested
+    if (_monitorProcessFinish) {
+        bool isForegroundProcessActive = _session->isForegroundProcessActive();
+        if (!_previousForegroundProcessName.isNull() && !isForegroundProcessActive) {
+            KNotification::event(_session->hasFocus() ? QStringLiteral("ProcessFinished") : QStringLiteral("ProcessFinishedHidden"),
+                                 i18n("The process '%1' has finished running in session '%2'", _previousForegroundProcessName, _session->nameTitle()),
+                                 QPixmap(),
+                                 QApplication::activeWindow(),
+                                 KNotification::CloseWhenWidgetActivated);
+        }
+        _previousForegroundProcessName = isForegroundProcessActive ? _session->foregroundProcessName() : QString();
+    }
+
     // do not forget icon
     updateSessionIcon();
 }
@@ -405,9 +449,10 @@ void SessionController::selectionChanged(const QString& selectedText)
 void SessionController::updateCopyAction(const QString& selectedText)
 {
     QAction* copyAction = actionCollection()->action(QStringLiteral("edit_copy"));
-
+    QAction* copyContextMenu = actionCollection()->action(QStringLiteral("edit_copy_contextmenu"));
     // copy action is meaningful only when some text is selected.
     copyAction->setEnabled(!selectedText.isEmpty());
+    copyContextMenu->setVisible(!selectedText.isEmpty());
 }
 
 void SessionController::updateWebSearchMenu()
@@ -472,7 +517,7 @@ void SessionController::handleWebShortcutAction()
 
     KUriFilterData filterData(action->data().toString());
 
-    if (KUriFilter::self()->filterUri(filterData, QStringList() << QStringLiteral("kurisearchfilter"))) {
+    if (KUriFilter::self()->filterUri(filterData, { QStringLiteral("kurisearchfilter") })) {
         const QUrl& url = filterData.uri();
         new KRun(url, QApplication::activeWindow());
     }
@@ -480,7 +525,7 @@ void SessionController::handleWebShortcutAction()
 
 void SessionController::configureWebShortcuts()
 {
-    KToolInvocation::kdeinitExec(QStringLiteral("kcmshell5"), QStringList() << QStringLiteral("webshortcuts"));
+    KToolInvocation::kdeinitExec(QStringLiteral("kcmshell5"), { QStringLiteral("webshortcuts") });
 }
 
 void SessionController::sendSignal(QAction* action)
@@ -510,30 +555,6 @@ void SessionController::toggleReadOnly()
     }
 }
 
-bool SessionController::eventFilter(QObject* watched , QEvent* event)
-{
-    if (event->type() == QEvent::FocusIn && watched == _view) {
-        // notify the world that the view associated with this session has been focused
-        // used by the view manager to update the title of the MainWindow widget containing the view
-        emit focused(this);
-
-        // when the view is focused, set bell events from the associated session to be delivered
-        // by the focused view
-
-        // first, disconnect any other views which are listening for bell signals from the session
-        disconnect(_session.data(), &Konsole::Session::bellRequest, nullptr, nullptr);
-        // second, connect the newly focused view to listen for the session's bell signal
-        connect(_session.data(), &Konsole::Session::bellRequest, _view.data(), &Konsole::TerminalDisplay::bell);
-
-        if ((_copyInputToAllTabsAction != nullptr) && _copyInputToAllTabsAction->isChecked()) {
-            // A session with "Copy To All Tabs" has come into focus:
-            // Ensure that newly created sessions are included in _copyToGroup!
-            copyInputToAllTabs();
-        }
-    }
-    return Konsole::ViewProperties::eventFilter(watched, event);
-}
-
 void SessionController::removeSearchFilter()
 {
     if (_searchFilter == nullptr) {
@@ -547,14 +568,14 @@ void SessionController::removeSearchFilter()
 
 void SessionController::setupSearchBar()
 {
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::unhandledMovementKeyPressed, this, &Konsole::SessionController::movementKeyFromSearchBarReceived);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::closeClicked, this, &Konsole::SessionController::searchClosed);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchFromClicked, this, &Konsole::SessionController::searchFrom);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findNextClicked, this, &Konsole::SessionController::findNextInHistory);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::findPreviousClicked, this, &Konsole::SessionController::findPreviousInHistory);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::highlightMatchesToggled , this , &Konsole::SessionController::highlightMatches);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchCaseToggled, this, &Konsole::SessionController::changeSearchMatch);
-    connect(_searchBar.data(), &Konsole::IncrementalSearchBar::matchRegExpToggled, this, &Konsole::SessionController::changeSearchMatch);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::unhandledMovementKeyPressed, this, &Konsole::SessionController::movementKeyFromSearchBarReceived);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::closeClicked, this, &Konsole::SessionController::searchClosed);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::searchFromClicked, this, &Konsole::SessionController::searchFrom);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::findNextClicked, this, &Konsole::SessionController::findNextInHistory);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::findPreviousClicked, this, &Konsole::SessionController::findPreviousInHistory);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::highlightMatchesToggled , this , &Konsole::SessionController::highlightMatches);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::matchCaseToggled, this, &Konsole::SessionController::changeSearchMatch);
+    connect(_searchBar, &Konsole::IncrementalSearchBar::matchRegExpToggled, this, &Konsole::SessionController::changeSearchMatch);
 }
 
 void SessionController::setShowMenuAction(QAction* action)
@@ -579,7 +600,7 @@ void SessionController::setupCommonActions()
     action->setIcon(QIcon::fromTheme(QStringLiteral("system-file-manager")));
 
     // Copy and Paste
-    action = KStandardAction::copy(this, SLOT(copy()), collection);
+    action = KStandardAction::copy(this, &SessionController::copy, collection);
 #ifdef Q_OS_MACOS
     // Don't use the Konsole::ACCEL const here, we really want the Command key (Qt::META)
     // TODO: check what happens if we leave it to Qt to assign the default?
@@ -590,6 +611,14 @@ void SessionController::setupCommonActions()
     // disabled at first, since nothing has been selected now
     action->setEnabled(false);
 
+    // We need a different QAction on the context menu because one will be disabled when there's no selection,
+    // other will be hidden.
+    action = collection->addAction(QStringLiteral("edit_copy_contextmenu"));
+    action->setText(i18n("Copy"));
+    action->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
+    action->setVisible(false);
+    connect(action, &QAction::triggered, this, &SessionController::copy);
+
     action = KStandardAction::paste(this, SLOT(paste()), collection);
     QList<QKeySequence> pasteShortcut;
 #ifdef Q_OS_MACOS
@@ -678,7 +707,7 @@ void SessionController::setupCommonActions()
     _codecAction->setIcon(QIcon::fromTheme(QStringLiteral("character-set")));
     collection->addAction(QStringLiteral("set-encoding"), _codecAction);
     _codecAction->setCurrentCodec(QString::fromUtf8(_session->codec()));
-    connect(_session.data(), &Konsole::Session::sessionCodecChanged, this, &Konsole::SessionController::updateCodecAction);
+    connect(_session, &Konsole::Session::sessionCodecChanged, this, &Konsole::SessionController::updateCodecAction);
     connect(_codecAction,
             QOverload<QTextCodec*>::of(&KCodecAction::triggered), this,
             &Konsole::SessionController::changeCodec);
@@ -696,7 +725,7 @@ void SessionController::setupExtraActions()
 
     // Rename Session
     QAction* action = collection->addAction(QStringLiteral("rename-session"), this, SLOT(renameSession()));
-    action->setText(i18n("&Rename Tab..."));
+    action->setText(i18n("&Current Tab Settings..."));
     action->setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
     collection->setDefaultShortcut(action, Konsole::ACCEL + Qt::ALT + Qt::Key_S);
 
@@ -747,6 +776,10 @@ void SessionController::setupExtraActions()
     action = collection->addAction(QStringLiteral("monitor-silence"), toggleAction);
     connect(action, &QAction::toggled, this, &Konsole::SessionController::monitorSilence);
 
+    toggleAction = new KToggleAction(i18n("Monitor for Process Finishing"), this);
+    action = collection->addAction(QStringLiteral("monitor-process-finish"), toggleAction);
+    connect(action, &QAction::toggled, this, &Konsole::SessionController::monitorProcessFinish);
+
     // Text Size
     action = collection->addAction(QStringLiteral("enlarge-font"), this, SLOT(increaseFontSize()));
     action->setText(i18n("Enlarge Font"));
@@ -859,22 +892,24 @@ void SessionController::editCurrentProfile()
     // NOTE bug311270: For to prevent the crash, the profile must be reset.
     if (!_editProfileDialog.isNull()) {
         // exists but not visible
-        delete _editProfileDialog.data();
+        _editProfileDialog->deleteLater();
     }
 
     _editProfileDialog = new EditProfileDialog(QApplication::activeWindow());
-    _editProfileDialog.data()->setProfile(SessionManager::instance()->sessionProfile(_session));
-    _editProfileDialog.data()->show();
+    _editProfileDialog->setProfile(SessionManager::instance()->sessionProfile(_session));
+    _editProfileDialog->show();
 }
 
 void SessionController::renameSession()
 {
     const QString &sessionLocalTabTitleFormat = _session->tabTitleFormat(Session::LocalTabTitle);
     const QString &sessionRemoteTabTitleFormat = _session->tabTitleFormat(Session::RemoteTabTitle);
+    const QColor &sessionTabColor = _session->color();
 
     QScopedPointer<RenameTabDialog> dialog(new RenameTabDialog(QApplication::activeWindow()));
     dialog->setTabTitleText(sessionLocalTabTitleFormat);
     dialog->setRemoteTabTitleText(sessionRemoteTabTitleFormat);
+    dialog->setColor(sessionTabColor);
 
     if (_session->isRemote()) {
         dialog->focusRemoteTabTitleText();
@@ -891,6 +926,7 @@ void SessionController::renameSession()
     if (result != 0) {
         const QString &tabTitle = dialog->tabTitleText();
         const QString &remoteTabTitle = dialog->remoteTabTitleText();
+        const QColor &tabColor = dialog->color();
 
         if (tabTitle != sessionLocalTabTitleFormat) {
             _session->setTabTitleFormat(Session::LocalTabTitle, tabTitle);
@@ -904,6 +940,12 @@ void SessionController::renameSession()
             emit tabRenamedByUser(true);
             snapshot();
         }
+
+        if (tabColor != sessionTabColor) {
+            _session->setColor(tabColor);
+            emit tabColoredByUser(true);
+            snapshot();
+        }
     }
 }
 
@@ -1084,8 +1126,12 @@ void SessionController::copyInputToAllTabs()
     // Find our window ...
     const KXmlGuiWindow* myWindow = findWindow(_view);
 
-    QSet<Session*> group =
-        QSet<Session*>::fromList(SessionManager::instance()->sessions());
+    const QList<Session *> sessionsList = SessionManager::instance()->sessions();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    QSet<Session*> group(sessionsList.begin(), sessionsList.end());
+#else
+    QSet<Session*> group = QSet<Session*>::fromList(sessionsList);
+#endif
     for (auto session : group) {
         // First, ensure that the session is removed
         // (necessary to avoid duplicates on addSession()!)
@@ -1100,6 +1146,7 @@ void SessionController::copyInputToAllTabs()
     _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
 
     snapshot();
+    emit copyInputChanged(this);
 }
 
 void SessionController::copyInputToSelectedTabs()
@@ -1114,7 +1161,13 @@ void SessionController::copyInputToSelectedTabs()
     QPointer<CopyInputDialog> dialog = new CopyInputDialog(_view);
     dialog->setMasterSession(_session);
 
-    QSet<Session*> currentGroup = QSet<Session*>::fromList(_copyToGroup->sessions());
+    const QList<Session*> sessionsList = _copyToGroup->sessions();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    QSet<Session*> currentGroup(sessionsList.begin(), sessionsList.end());
+#else
+    QSet<Session*> currentGroup = QSet<Session*>::fromList(sessionsList);
+#endif
+
     currentGroup.remove(_session);
 
     dialog->setChosenSessions(currentGroup);
@@ -1141,6 +1194,7 @@ void SessionController::copyInputToSelectedTabs()
         _copyToGroup->setMasterStatus(_session, true);
         _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
         snapshot();
+        emit copyInputChanged(this);
     }
 }
 
@@ -1150,8 +1204,14 @@ void SessionController::copyInputToNone()
         return;
     }
 
-    QSet<Session*> group =
-        QSet<Session*>::fromList(SessionManager::instance()->sessions());
+    // Once Qt5.14+ is the mininum, change to use range constructors
+    const QList<Session*> groupList = SessionManager::instance()->sessions();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    QSet<Session*> group(groupList.begin(), groupList.end());
+#else
+    QSet<Session*> group = QSet<Session*>::fromList(groupList);
+#endif
+
     for (auto iterator : group) {
         Session* session = iterator;
 
@@ -1162,6 +1222,7 @@ void SessionController::copyInputToNone()
     delete _copyToGroup;
     _copyToGroup = nullptr;
     snapshot();
+    emit copyInputChanged(this);
 }
 
 void SessionController::searchClosed()
@@ -1218,7 +1279,7 @@ void SessionController::listenForScreenWindowUpdates()
     connect(_view->screenWindow(), &Konsole::ScreenWindow::outputChanged, this, &Konsole::SessionController::updateSearchFilter);
     connect(_view->screenWindow(), &Konsole::ScreenWindow::scrolled, this, &Konsole::SessionController::updateSearchFilter);
     connect(_view->screenWindow(),
-            &Konsole::ScreenWindow::currentResultLineChanged, _view.data(),
+            &Konsole::ScreenWindow::currentResultLineChanged, _view,
             QOverload<>::of(&Konsole::TerminalDisplay::update));
 
     _listenForScreenWindowUpdates = true;
@@ -1258,15 +1319,15 @@ void SessionController::enableSearchBar(bool showSearchBar)
 
     _searchBar->setVisible(showSearchBar);
     if (showSearchBar) {
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchChanged, this, &Konsole::SessionController::searchTextChanged);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchReturnPressed, this, &Konsole::SessionController::findPreviousInHistory);
-        connect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchShiftPlusReturnPressed, this, &Konsole::SessionController::findNextInHistory);
+        connect(_searchBar, &Konsole::IncrementalSearchBar::searchChanged, this, &Konsole::SessionController::searchTextChanged);
+        connect(_searchBar, &Konsole::IncrementalSearchBar::searchReturnPressed, this, &Konsole::SessionController::findPreviousInHistory);
+        connect(_searchBar, &Konsole::IncrementalSearchBar::searchShiftPlusReturnPressed, this, &Konsole::SessionController::findNextInHistory);
     } else {
-        disconnect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchChanged, this,
+        disconnect(_searchBar, &Konsole::IncrementalSearchBar::searchChanged, this,
                    &Konsole::SessionController::searchTextChanged);
-        disconnect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchReturnPressed, this,
+        disconnect(_searchBar, &Konsole::IncrementalSearchBar::searchReturnPressed, this,
                    &Konsole::SessionController::findPreviousInHistory);
-        disconnect(_searchBar.data(), &Konsole::IncrementalSearchBar::searchShiftPlusReturnPressed, this,
+        disconnect(_searchBar, &Konsole::IncrementalSearchBar::searchShiftPlusReturnPressed, this,
                    &Konsole::SessionController::findNextInHistory);
         if ((!_view.isNull()) && (_view->screenWindow() != nullptr)) {
             _view->screenWindow()->setCurrentResultLine(-1);
@@ -1504,9 +1565,9 @@ void SessionController::print_screen()
     QPointer<QPrintDialog> dialog = new QPrintDialog(&printer, _view);
     auto options = new PrintOptions();
 
-    dialog->setOptionTabs(QList<QWidget*>() << options);
+    dialog->setOptionTabs({options});
     dialog->setWindowTitle(i18n("Print Shell"));
-    connect(dialog.data(),
+    connect(dialog,
             QOverload<>::of(&QPrintDialog::accepted),
             options,
             &Konsole::PrintOptions::saveSettings);
@@ -1578,6 +1639,10 @@ void SessionController::monitorSilence(bool monitor)
 {
     _session->setMonitorSilence(monitor);
 }
+void SessionController::monitorProcessFinish(bool monitor)
+{
+    _monitorProcessFinish = monitor;
+}
 void SessionController::updateSessionIcon()
 {
     // If the default profile icon is being used, don't put it on the tab
@@ -1589,16 +1654,7 @@ void SessionController::updateSessionIcon()
     }
     _sessionIcon = QIcon::fromTheme(_sessionIconName);
 
-    // Visualize that the session is broadcasting to others
-    if ((_copyToGroup != nullptr) && _copyToGroup->sessions().count() > 1) {
-        // Master Mode: set different icon, to warn the user to be careful
-        setIcon(*_broadcastIcon);
-    } else {
-        if (!_keepIconUntilInteraction) {
-            // Not in Master Mode: use normal icon
-            setIcon(_sessionIcon);
-        }
-    }
+    setIcon(_sessionIcon);
 }
 
 void SessionController::updateReadOnlyActionStates()
@@ -1612,7 +1668,7 @@ void SessionController::updateReadOnlyActionStates()
     auto updateActionState = [this, readonly](const QString &name) {
         QAction *action = actionCollection()->action(name);
         if (action != nullptr) {
-            action->setEnabled(!readonly);
+            action->setVisible(!readonly);
         }
     };
 
@@ -1643,6 +1699,11 @@ bool SessionController::isReadOnly() const
     }
 }
 
+bool SessionController::isCopyInputActive() const
+{
+    return ((_copyToGroup != nullptr) && _copyToGroup->sessions().count() > 1);
+}
+
 void SessionController::sessionAttributeChanged()
 {
     if (_sessionIconName != _session->iconName()) {
@@ -1663,21 +1724,20 @@ void SessionController::sessionAttributeChanged()
     }
 
     setTitle(title);
+    setColor(_session->color());
     emit rawTitleChanged();
 }
 
 void SessionController::sessionReadOnlyChanged() {
-    // Trigger icon update
-    sessionAttributeChanged();
-
     updateReadOnlyActionStates();
 
     // Update all views
     const QList<TerminalDisplay *> viewsList = session()->views();
     for (TerminalDisplay *terminalDisplay : viewsList) {
-        if (terminalDisplay != _view.data()) {
+        if (terminalDisplay != _view) {
             terminalDisplay->updateReadOnlyState(isReadOnly());
         }
+        emit readOnlyChanged(this);
     }
 }
 
@@ -1704,11 +1764,19 @@ void SessionController::showDisplayContextMenu(const QPoint& position)
         auto contentSeparator = new QAction(popup);
         contentSeparator->setSeparator(true);
 
+        // We don't actually use this shortcut, but we need to display it for consistency :/
+        QAction *copy = actionCollection()->action(QStringLiteral("edit_copy_contextmenu"));
+#ifdef Q_OS_MACOS
+        copy->setShortcut(Qt::META + Qt::Key_C);
+#else
+        copy->setShortcut(Konsole::ACCEL + Qt::SHIFT + Qt::Key_C);
+#endif
         // prepend content-specific actions such as "Open Link", "Copy Email Address" etc.
         QSharedPointer<Filter::HotSpot> hotSpot = _view->filterActions(position);
-        if (hotSpot) {
+        if (hotSpot != nullptr) {
             popup->insertActions(popup->actions().value(0, nullptr), hotSpot->actions() << contentSeparator );
-
+            popup->addAction(contentSeparator);
+            hotSpot->setupMenu(popup.data());
         }
 
         // always update this submenu before showing the context menu,
@@ -1740,6 +1808,9 @@ void SessionController::showDisplayContextMenu(const QPoint& position)
         if ((chosen != nullptr) && chosen->objectName() == QLatin1String("close-session")) {
             chosen->trigger();
         }
+
+        // Remove the Accelerator for the copy shortcut so we don't have two actions with same shortcut.
+        copy->setShortcut({});
     } else {
         qCDebug(KonsoleDebug) << "Unable to display popup menu for session"
                    << _session->title(Session::NameRole)
@@ -1753,26 +1824,9 @@ void SessionController::movementKeyFromSearchBarReceived(QKeyEvent *event)
     setSearchStartToWindowCurrentLine();
 }
 
-void SessionController::sessionStateChanged(int state)
+void SessionController::sessionNotificationsChanged(Session::Notification notification, bool enabled)
 {
-    if (state == _previousState) {
-        return;
-    }
-
-    if (state == NOTIFYACTIVITY) {
-        setIcon(*_activityIcon);
-        _keepIconUntilInteraction = true;
-    } else if (state == NOTIFYSILENCE) {
-        setIcon(*_silenceIcon);
-        _keepIconUntilInteraction = true;
-    } else if (state == NOTIFYBELL) {
-        setIcon(*_bellIcon);
-        _keepIconUntilInteraction = true;
-    } else if (state == NOTIFYNORMAL) {
-        updateSessionIcon();
-    }
-
-    _previousState = state;
+    emit notificationChanged(this, notification, enabled);
 }
 
 void SessionController::zmodemDownload()
