@@ -34,6 +34,7 @@
 #include "ViewProperties.h"
 #include "Profile.h"
 #include "Enumeration.h"
+#include "Session.h"
 
 class QAction;
 class QTextCodec;
@@ -141,9 +142,6 @@ public:
     bool confirmClose() const override;
     virtual bool confirmForceClose() const;
 
-    // Reimplemented to watch for events happening to the view
-    bool eventFilter(QObject *watched, QEvent *event) override;
-
     /** Returns the set of all controllers that exist. */
     static QSet<SessionController *> allControllers()
     {
@@ -153,6 +151,7 @@ public:
     /* Returns true if called within a KPart; false if called within Konsole. */
     bool isKonsolePart() const;
     bool isReadOnly() const;
+    bool isCopyInputActive() const;
 
 Q_SIGNALS:
     /**
@@ -160,7 +159,7 @@ Q_SIGNALS:
      * This can be used by other classes to plug the controller's actions into a window's
      * menus.
      */
-    void focused(SessionController *controller);
+    void viewFocused(SessionController *controller);
 
     void rawTitleChanged();
 
@@ -175,6 +174,11 @@ Q_SIGNALS:
      */
     void tabRenamedByUser(bool renamed) const;
 
+    /**
+     * Emitted when the user changes the tab color.
+     */
+    void tabColoredByUser(bool set) const;
+
 public Q_SLOTS:
     /**
      * Issues a command to the session to navigate to the specified URL.
@@ -243,6 +247,7 @@ private Q_SLOTS:
     void clearHistoryAndReset();
     void monitorActivity(bool monitor);
     void monitorSilence(bool monitor);
+    void monitorProcessFinish(bool monitor);
     void renameSession();
     void switchProfile(const Profile::Ptr &profile);
     void handleWebShortcutAction();
@@ -258,7 +263,7 @@ private Q_SLOTS:
     void updateCodecAction(QTextCodec *codec);
     void showDisplayContextMenu(const QPoint &position);
     void movementKeyFromSearchBarReceived(QKeyEvent *event);
-    void sessionStateChanged(int state);
+    void sessionNotificationsChanged(Session::Notification notification, bool enabled);
     void sessionAttributeChanged();
     void sessionReadOnlyChanged();
     void searchTextChanged(const QString &text);
@@ -266,6 +271,7 @@ private Q_SLOTS:
 
     void updateFilterList(Profile::Ptr profile); // Called when the profile has changed, so we might need to change the list of filters
 
+    void viewFocusChangeHandler(bool focused);
     void interactionHandler();
     void snapshot(); // called periodically as the user types
     // to take a snapshot of the state of the
@@ -315,7 +321,6 @@ private:
 
     QIcon _sessionIcon;
     QString _sessionIconName;
-    int _previousState;
 
     RegExpFilter *_searchFilter;
     UrlFilter *_urlFilter;
@@ -340,8 +345,6 @@ private:
     bool _listenForScreenWindowUpdates;
     bool _preventClose;
 
-    bool _keepIconUntilInteraction;
-
     QString _selectedText;
 
     QAction *_showMenuAction;
@@ -356,6 +359,9 @@ private:
 
     QString _searchText;
     QPointer<IncrementalSearchBar> _searchBar;
+
+    QString _previousForegroundProcessName;
+    bool _monitorProcessFinish;
 };
 inline bool SessionController::isValid() const
 {
