@@ -93,8 +93,7 @@ public Q_SLOTS:
     void sendText(const QString &text) override;
     void sendKeyEvent(QKeyEvent *) override;
     void sendMouseEvent(int buttons, int column, int line, int eventType) override;
-    void focusLost() override;
-    void focusGained() override;
+    void focusChanged(bool focused) override;
 
 protected:
     // reimplemented from Emulation
