@@ -42,6 +42,10 @@ private Q_SLOTS:
     void testColorSpaceDefault();
     void testColorSpaceSystem_data();
     void testColorSpaceSystem();
+    void testColorSpaceRGB_data();
+    void testColorSpaceRGB();
+    void testColor256_data();
+    void testColor256();
 
 private:
     static const ColorEntry DefaultColorTable[];
